/* *
 *   Translated to Java by Gaurav Chhabra
 *
 *   libprayer: A full featured Muslim Prayer Times calculator
 *   Copyright (C) 2003 Thamer Mahmoud (neokuwait@hotmail.com)
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later
 *   version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *   02111-1307 USA
 *
 * */


public class PrayerTimeCalculator
{
	public static final double PI			= 3.1415926535 ;
	public static final double KAABA_LAT	= 21.423333 ;
	public static final double KAABA_LONG	= 39.823333 ;

	public void PrayerTimeCalculator()
	{
	}

	public void getPrayerTimes(PrayerTimeLocation ptLoc, PrayerTimeMethod ptMethod, PrayerTimeDate ptDate, Prayer[] pt)
	{
		 MInteger nDay		= new MInteger();
		 MInteger lastDay	= new MInteger();
		 MInteger nextDay	= new MInteger();
		 getDayInfo (ptDate,nDay,lastDay,nextDay);
		 getPrayerTimesByDay (ptLoc,ptMethod,nDay.getInt(),lastDay.getInt(),pt);
	}



	private void getPrayerTimesByDay ( PrayerTimeLocation ptLoc, PrayerTimeMethod ptMethod, int nDay, int lastDay, Prayer[] pt)
	{
		 int i;
		 double eot;
		 double dec;
		 double th;
		 double shMg;
		 double fj;
		 double is;
		 double ar;
		 double lat;
		 double lon;
		 double gmt;
		 double[] tempPrayer = {0.00,0.00,0.00,0.00,0.00,0.00};


		 /* fill info variables */
		 lat = ptLoc.getDegreeLat();
		 lon = ptLoc.getDegreeLong();
		 gmt = ptLoc.getGMTDiff() * 15;
		 eot = timeEquation (nDay, lastDay);
		 dec = sunDeclination (nDay);

		/** First Step: Get Prayer Times results for this day of year
		 *  and this location. The results are NOT the actual prayer
		 *  times
		 */

		th		= getThuhr  (lon, gmt, eot);
		shMg	= getShoMag (lat, dec, ptLoc.getSeaLevel());
		fj		= getFajIsh (lat, dec, ptMethod.getFajrAngle());
		is		= getFajIsh (lat, dec, ptMethod.getIshaaAngle());
		ar		= getAssr   (lat, dec, ptMethod.getMathhab());


		/** Second Step A: Calculate all salat times as Base-10 numbers
		 *   in Normal circumstances
		 */

		/* Fajr */
		if (ptMethod.getFajrInv() != 0)
		{
			double interval = ptMethod.getFajrInv() / 60.0;
			tempPrayer[0]	= th - shMg - interval;
		}
		else if (fj == 0)
			tempPrayer[0] = 0;
		else
			tempPrayer[0] = th - fj;

     tempPrayer[1] = th - shMg;
     tempPrayer[2] = th;
     tempPrayer[3] = th + ar;
     tempPrayer[4] = th + shMg;

     /* Ishaa */
     if (is == 0)
	  tempPrayer[5] = 0;
     else tempPrayer[5] = th + is;
     /* Ishaa Interval */
     if (ptMethod.getIshaaInv() != 0)
	 {
	  double interval = ptMethod.getIshaaInv() / 60.0;
	  tempPrayer[5] = th + shMg + interval;
     }


     /* Second Step B: Calculate all salat times as Base-10 numbers
      * in Extreme Latitudes (if set)
	  */

     /* Reset status of extreme switches */
     for (i=0; i<6; i++)
	  pt[i].setIsExtreme(0);

     if (ptMethod.getExtreme() != 0)
     {
		 double exeot;
		 double exdec;
		 double exTh;
		 double exShMg;
		 double exFj;
		 double exIs;
		 double exAr;
		 double exIm;

		  /* Nearest Latitude (Method.nearestLat) */
		  if (ptMethod.getExtreme() <= 3)
		  {
			exFj		= getFajIsh(ptMethod.getNearestLat(), dec, ptMethod.getFajrAngle());
			exIm		= getFajIsh(ptMethod.getNearestLat(), dec, ptMethod.getImsaakAngle());
			exIs		= getFajIsh(ptMethod.getNearestLat(), dec, ptMethod.getIshaaAngle());
			exAr		= getAssr(ptMethod.getNearestLat()  , dec, ptMethod.getMathhab());
			exShMg		= getShoMag(ptMethod.getNearestLat(), dec, ptLoc.getSeaLevel());

			switch(ptMethod.getExtreme())
			{
			   case 1: /* All salat Always: Nearest Latitude */
						tempPrayer[0] = th - exFj;
						tempPrayer[1] = th - exShMg;
						tempPrayer[3] = th + exAr;
						tempPrayer[4] = th + exShMg;
						tempPrayer[5] = th + exIs;
						pt[0].setIsExtreme(1);
						pt[1].setIsExtreme(1);
						pt[3].setIsExtreme(1);
						pt[4].setIsExtreme(1);
						pt[5].setIsExtreme(1);
						break;

			   case 2: /* Fajr Ishaa Always: Nearest Latitude */
						tempPrayer[0] = th - exFj;
						tempPrayer[5] = th + exIs;
						pt[0].setIsExtreme(1);
						pt[5].setIsExtreme(1);
						break;

			   case 3:/* Fajr Ishaa if invalid: Nearest Latitude */
						if (tempPrayer[0] <= 0)
						{
							tempPrayer[0] = th - exFj;
							pt[0].setIsExtreme(1);
						}
						if (tempPrayer[5] <= 0)
						{
							tempPrayer[5] = th + exIs;
							pt[5].setIsExtreme(1);
						}
						break;
			}
		} /* Nearest latitude */


		/* Nearest Good Day */
		if ((ptMethod.getExtreme() > 3) && (ptMethod.getExtreme() <= 5))
		{
			int nGoodDay = 0;

			/* Start by getting last or next nearest Good Day */
			for(i=0; i <= lastDay; i++)
			{
				/* last closest day */
				nGoodDay = nDay - i;
				exeot = timeEquation (nGoodDay, lastDay);
				exdec = sunDeclination(nGoodDay);
				exTh = getThuhr(lon, gmt, exeot);
				exFj = getFajIsh(lat, exdec, ptMethod.getFajrAngle());
				exIs = getFajIsh(lat, exdec, ptMethod.getIshaaAngle());

				if ((exFj > 0) && (exIs > 0))
					break; /* loop */


				/* Next closest day */
				nGoodDay = nDay + i;
				exdec = sunDeclination(nGoodDay);
				exTh = getThuhr(lon, gmt, exeot);
				exFj = getFajIsh(lat, exdec, ptMethod.getFajrAngle());
				exIs = getFajIsh(lat, exdec, ptMethod.getIshaaAngle());

				if ((exFj > 0) && (exIs > 0))
					break;
			}

			exeot	= timeEquation	(nGoodDay, lastDay);
			exdec	= sunDeclination (nGoodDay);
			exTh	= getThuhr (lon, gmt, exeot);
			exFj	= getFajIsh (lat, exdec, ptMethod.getFajrAngle());
			exIs	= getFajIsh (lat, exdec, ptMethod.getIshaaAngle());
			exShMg	= getShoMag (lat, exdec, ptLoc.getSeaLevel());
			exAr	= getAssr (lat, exdec, ptMethod.getMathhab());

			switch(ptMethod.getExtreme())
			{
				case 4: /* All salat Always: Nearest Day */
						tempPrayer[0] = exTh - exFj;
						tempPrayer[1] = exTh - exShMg;
						tempPrayer[2] = exTh;
						tempPrayer[3] = exTh + exAr;
						tempPrayer[4] = exTh + exShMg;
						tempPrayer[5] = exTh + exIs;
						for (i=0; i<6; i++)
						 pt[i].setIsExtreme(1);
						break;
				case 5: /* Fajr Ishaa if invalid:: Nearest Day */
						if (tempPrayer[0] <= 0) {
						 tempPrayer[0] = exTh - exFj;
						 pt[0].setIsExtreme(1);
						}
						if (tempPrayer[5] <= 0) {
						 tempPrayer[5] = exTh + exIs;
						 pt[5].setIsExtreme(1);
						}
						break;
			} /* end switch */
		} /* end nearest day */

		/*1/7th of Night */
		if (ptMethod.getExtreme() == 6 || ptMethod.getExtreme() == 7)
		{
		   double allInterval = 24 - (th - shMg);
		   allInterval = allInterval + (12 - (th + shMg));

		   switch(ptMethod.getExtreme())
		   {
			   case 6: /*Fajr Ishaa Always: 1/7th of Night */
					tempPrayer[0] = (th - shMg) - ((1/7.0) * allInterval);
					pt[0].setIsExtreme(1);
					tempPrayer[5] = ((1/7.0) * allInterval) + (th + shMg);
					pt[5].setIsExtreme(1);
					break;

			   case 7: /*Fajr Ishaa if invalid: 1/7th of Night */
					if (tempPrayer[0] <= 0)
					{
						tempPrayer[0] = (th - shMg) - ((1/7.0) * allInterval);
						pt[0].setIsExtreme(1);
					}
					if (tempPrayer[5] <= 0)
					{
						tempPrayer[5] = ((1/7.0) * allInterval) + (th + shMg);
						pt[5].setIsExtreme(1);
					}
					break;
		   }
		} /* end 1/7th of Night */

		/* n Minutes from Shorooq Maghrib */
		if (ptMethod.getExtreme() == 8 || ptMethod.getExtreme() == 9)
		{
			switch(ptMethod.getExtreme())
			{
				case 8: /* Minutes from Shorooq/Maghrib Always */
						tempPrayer[0] = th - shMg;
						pt[0].setIsExtreme(1);
						tempPrayer[5] = th + shMg;
						pt[5].setIsExtreme(1);
						break;

				case 9: /* Minutes from Shorooq/Maghrib if invalid */
						if (tempPrayer[0] <= 0)
						{
							tempPrayer[0] = th - shMg;
							pt[0].setIsExtreme(1);
						}
						if (tempPrayer[5] <= 0)
						{
							tempPrayer[5] = th + shMg;
							pt[5].setIsExtreme(1);
						}
						break;
			} /* end switch */
		} /* end n Minutes */
	} /* End Extreme */

    /* Third and Final Step: Fill the Prayer array and do decimal
     * to minutes conversion
	 */

	for (i=0; i<6; i++)
	{
		tempPrayer[i] += (pt[i].getOffset() / 60.0);
		base6hm(tempPrayer[i], pt[i], ptLoc.getDST()); // Check this out !!!
	}

}


public void getImsaak (PrayerTimeLocation ptLoc, PrayerTimeMethod ptMethod, PrayerTimeDate ptDate, Prayer pt)
{
	PrayerTimeMethod tmpPTMethod = new PrayerTimeMethod();

	MInteger nDay		= new MInteger();
	MInteger lastDay	= new MInteger();
	MInteger nextDay	= new MInteger();

	Prayer[] temp = new Prayer[6];
	for(int iCounter = 0; iCounter < 6; iCounter++)
	{
		temp[iCounter] = new Prayer();
		temp[iCounter].setMinute(0);
		temp[iCounter].setHour(0);
		temp[iCounter].setIsExtreme(0);
		temp[iCounter].setOffset(0);
	}

	tmpPTMethod.setFajrAngle(ptMethod.getFajrAngle());
	tmpPTMethod.setIshaaAngle(ptMethod.getIshaaAngle());
	tmpPTMethod.setImsaakAngle(ptMethod.getImsaakAngle());
	tmpPTMethod.setFajrInv(ptMethod.getFajrInv());
	tmpPTMethod.setIshaaInv(ptMethod.getIshaaInv());
	tmpPTMethod.setImsaakInv(ptMethod.getImsaakInv());
	tmpPTMethod.setMathhab(ptMethod.getMathhab());
	tmpPTMethod.setNearestLat(ptMethod.getNearestLat());
	tmpPTMethod.setExtreme(ptMethod.getExtreme());


    if (ptMethod.getFajrInv() != 0)
	{
	  if (ptMethod.getImsaakInv() == 0)
	       tmpPTMethod.setImsaakInv(10);
	  tmpPTMethod.setFajrInv(tmpPTMethod.getFajrInv() + ptMethod.getImsaakInv());
    }
	else if (ptMethod.getImsaakInv() != 0)
	{
	  /* use an inv even if al-Fajr is computed (Indonesia?) */
	  temp[0].setOffset(temp[0].getOffset() + (ptMethod.getImsaakInv() * -1));
    }
	else  tmpPTMethod.setFajrAngle(tmpPTMethod.getFajrAngle() + ptMethod.getImsaakAngle());

	getDayInfo ( ptDate, nDay, lastDay, nextDay);
    getPrayerTimesByDay( ptLoc, tmpPTMethod, (nDay).getInt(), (lastDay).getInt(), temp);

     /* In case of an extreme Fajr time calculation
      * use intervals for Imsaak and compute again
	  */
     if (temp[0].getIsExtreme() != 0 && ptMethod.getImsaakInv() != 0)
     {
		tmpPTMethod.setImsaakInv(10);
		getPrayerTimesByDay( ptLoc, tmpPTMethod, (nDay).getInt(), (lastDay).getInt(), temp);
     }
     pt.setHour(temp[0].getHour());
     pt.setMinute(temp[0].getMinute());
     pt.setIsExtreme(temp[0].getIsExtreme());
}

public void getNextDayImsaak (PrayerTimeLocation ptLoc, PrayerTimeMethod ptMethod, PrayerTimeDate ptDate, Prayer pt)
{
	/* increment for next day */
	Prayer temppt = new Prayer();
	temppt.setMinute(0);
	temppt.setHour(0);
	temppt.setIsExtreme(0);
	temppt.setOffset(0);

     PrayerTimeDate tempd = new PrayerTimeDate();
     tempd.setDay(ptDate.getDay());
     tempd.setMonth(ptDate.getMonth());
     tempd.setYear(ptDate.getYear());
     tempd.setDay(tempd.getDay() + 1);

     getImsaak (ptLoc, ptMethod, tempd, temppt);

     pt.setHour(temppt.getHour());
	 pt.setMinute(temppt.getMinute());
     pt.setIsExtreme(temppt.getIsExtreme());
}

public void getNextDayFajr (PrayerTimeLocation ptLoc, PrayerTimeMethod ptMethod, PrayerTimeDate ptDate, Prayer pt)
{
	MInteger lastDay = new MInteger();
	MInteger nextDay = new MInteger();
	MInteger nDay	= new MInteger();
	Prayer[] temp = new Prayer[6];
	for(int iCounter = 0; iCounter < 6; iCounter++)
	{
	temp[iCounter] = new Prayer();
	temp[iCounter].setMinute(0);
	temp[iCounter].setHour(0);
	temp[iCounter].setIsExtreme(0);
	temp[iCounter].setOffset(0);
	}

	getDayInfo ( ptDate, nDay, lastDay, nextDay);
	getPrayerTimesByDay( ptLoc, ptMethod, (nextDay).getInt(), (lastDay).getInt(), temp);

     pt.setHour(temp[0].getHour());
     pt.setMinute(temp[0].getMinute());
     pt.setIsExtreme(temp[0].getIsExtreme());

}

private void base6hm(double sTime, Prayer pt, int dst)
{
    double temp = (sTime - (int)Math.floor(sTime)) * 60;
	if (sTime<0)
	{
		while (sTime < 0)
			sTime = 24 + sTime;
	}

    sTime += dst;
    if (sTime > 23)
		sTime = sTime - (24 * (int)( sTime / 24));	// replacement for fmod in C

    pt.setHour((int)sTime);
    pt.setMinute((int)temp);
}


private double getThuhr(double Lon, double ref, double eot)
{
     return (12 + ((ref-Lon)/15.0) - ((eot/60.0)));
}


private double getShoMag(double Lat, double dec, double seaLevel)
{
     double part1 = Math.sin(deg2Rad(Lat)) * Math.sin(dec);
     double part2 = (Math.sin(deg2Rad(-0.8333-0.0347 * (Math.pow(seaLevel,0.5)))) - part1);
     double part3 = Math.cos(deg2Rad(Lat)) * Math.cos(dec);

     return (0.0667 * (Math.acos(part2 / part3) / deg2Rad(0)));
}


private double getFajIsh(double Lat, double dec, double Ang)
{
     double part1 = Math.cos(deg2Rad(Lat)) * Math.cos(dec);
     double part2 = -Math.sin(deg2Rad(Ang)) - Math.sin(deg2Rad(Lat)) * Math.sin(dec);

     double test = part2 / part1;
     if (test <= -1)
	  return 0;

     return (0.0667 * (Math.acos(part2 / part1) / deg2Rad(0)));
}


private double getAssr(double Lat, double dec, int mathhab)
{
     double part1;
	 double part2;
	 double part3;
	 double part4;

     part1 = mathhab + Math.tan(deg2Rad(Lat) - dec);
     if (Lat < 0)
	  part1 = mathhab - Math.tan(deg2Rad(Lat) - dec);

     part2 = (PI/2.0) - Math.atan(part1);
     part3 = Math.sin(part2) - Math.sin(deg2Rad(Lat)) * Math.sin(dec);
     part4 = (part3 / (Math.cos(deg2Rad(Lat)) * Math.cos(dec)));

     return (0.0667 * Math.acos(part4) / deg2Rad(0));
}


private double timeEquation (int nDay, double lastday)
{
     double eot;
     double t = 360 * (nDay-81.0) / (double)lastday;
     t = deg2Rad(t);
     eot = 9.87 * Math.sin(2*t) - 7.53 * Math.cos(t) - 1.5 * Math.sin(t);

     return eot;
}

private double sunDeclination (int nDay)
{
     double dec = deg2Rad(23.45) * Math.sin(deg2Rad(.9836 * (284 + nDay)));
     return dec;
}


/* Convert Degrees to Radians (and vice-versa if n==0) */
private double deg2Rad(double n)
{
     if (n==0)
	  return (PI/180.0);
     else return n * (PI/180.0);
}


private int getDayofYear(int year, int month, int day)
{
     int i;
     int isLeap = (((year & 3) == 0) && ((year % 100) != 0 || (year % 400) == 0)) == true ? 1 : 0;

     char[][] dayList = {
	  {0,31,28,31,30,31,30,31,31,30,31,30,31},
	  {0,31,29,31,30,31,30,31,31,30,31,30,31}
     };

     for (i=1; i<month; i++)
	  day += dayList[isLeap][i];


     return day;
}


private double dms2Decimal(int deg, int min, int sec, char dir)
{
     double sum = deg + ((min/60.0)+(sec/3600.0));
     if (dir == 'S' || dir == 'W' || dir == 's' || dir == 'w')
	  return sum * (-1.0);
     return sum;
}

public void decimal2Dms(double decimal, MInteger deg, MInteger min, MInteger sec)
{
	double tempmin;
	double tempsec;
	double n1;
	double n2;
	double n3;
	double test;
	tempmin = (decimal >= 0 ? (decimal - Math.floor(decimal)) * 60.0 : (decimal - Math.ceil(decimal)) * 60.0) ;
	tempsec = (tempmin >= 0 ? (tempmin - Math.floor(tempmin)) * 60.0 : (tempmin - Math.ceil(tempmin)) * 60.0) ;

	/* Rounding seconds */
	n1 = (decimal >= 0 ? Math.floor(decimal) : Math.ceil(decimal));
	n2 = (tempmin >= 0 ? Math.floor(tempmin) : Math.ceil(tempmin));

	if(n1 < 0)
		n3 = Math.ceil(tempsec);
	else
		n3 = Math.floor(tempsec);

	test = tempsec - n3;

	if(test > 0.5)
		n3++;

	if (test < 0 && test < -0.5)
		n3--;

	if (n3 == 60)
	{
		n3 = 0;
		n2++;
	}

	if (n3 == -60)
	{
		n3 = 0;
		n2--;
	}

	deg.setInt((int)n1);
	min.setInt((int)n2);
	sec.setInt((int)n3);
}

private void getDayInfo ( PrayerTimeDate ptDate, MInteger nDay, MInteger lastDay, MInteger nextDay)
{
     int d;
	 int ld;
	 int nd;
     d = getDayofYear(ptDate.getYear(), ptDate.getMonth(), ptDate.getDay());
     ld = getDayofYear(ptDate.getYear(), 12, 31);
     nd = d + 1;
     if(nd > ld)
	  nd=1;
     nDay.setInt(d);
     lastDay.setInt(ld);
     nextDay.setInt(nd);
}

public void getMethod(int n, PrayerTimeMethod ptMethod)
{
     switch(n)
     {
		case 0: /* none. Set to default or 0 */
				ptMethod.setFajrAngle(0.0);
				ptMethod.setIshaaAngle(0.0);
				ptMethod.setImsaakAngle(1.5);
				ptMethod.setFajrInv(0);
				ptMethod.setIshaaInv(0);
				ptMethod.setImsaakInv(0);
				ptMethod.setMathhab(1);
				ptMethod.setNearestLat(48.5);
				ptMethod.setExtreme(7);
				break;

		case 1: /* Egyptian General Authority of Survey */
				ptMethod.setFajrAngle(20);
				ptMethod.setIshaaAngle(18);
				ptMethod.setImsaakAngle(1.5);
				ptMethod.setFajrInv(0);
				ptMethod.setIshaaInv(0);
				ptMethod.setImsaakInv(0);
				ptMethod.setMathhab(1);
				ptMethod.setNearestLat(48.5);
				ptMethod.setExtreme(7);
				break;

     case 2: /* University of Islamic Sciences, Karachi (Shaf'i) */
			  ptMethod.setFajrAngle(18);
			  ptMethod.setIshaaAngle(18);
			  ptMethod.setImsaakAngle(1.5);
			  ptMethod.setFajrInv(0);
			  ptMethod.setIshaaInv(0);
			  ptMethod.setImsaakInv(0);
			  ptMethod.setMathhab(1);
			  ptMethod.setNearestLat(48.5);
			  ptMethod.setExtreme(7);
			  break;

     case 3: /* University of Islamic Sciences, Karachi (Hanafi) */
			  ptMethod.setFajrAngle(18);
			  ptMethod.setIshaaAngle(18);
			  ptMethod.setImsaakAngle(1.5);
			  ptMethod.setFajrInv(0);
			  ptMethod.setIshaaInv(0);
			  ptMethod.setImsaakInv(0);
			  ptMethod.setMathhab(2);
			  ptMethod.setNearestLat(48.5);
			  ptMethod.setExtreme(7);
			  break;

     case 4: /* Islamic Society of North America */
			  ptMethod.setFajrAngle(15);
			  ptMethod.setIshaaAngle(15);
			  ptMethod.setImsaakAngle(1.5);
			  ptMethod.setFajrInv(0);
			  ptMethod.setIshaaInv(0);
			  ptMethod.setImsaakInv(0);
			  ptMethod.setMathhab(1);
			  ptMethod.setNearestLat(48.5);
			  ptMethod.setExtreme(7);
			  break;

     case 5: /* Muslim World League (MWL) */
			  ptMethod.setFajrAngle(18);
			  ptMethod.setIshaaAngle(17);
			  ptMethod.setImsaakAngle(1.5);
			  ptMethod.setFajrInv(0);
			  ptMethod.setIshaaInv(0);
			  ptMethod.setImsaakInv(0);
			  ptMethod.setMathhab(1);
			  ptMethod.setNearestLat(48.5);
			  ptMethod.setExtreme(7);
			  break;

     case 6: /* Umm Al-Qurra */
			  ptMethod.setFajrAngle(19);
			  ptMethod.setIshaaAngle(0.0);
			  ptMethod.setImsaakAngle(1.5);
			  ptMethod.setFajrInv(0);
			  ptMethod.setIshaaInv(90);
			  ptMethod.setImsaakInv(0);
			  ptMethod.setMathhab(1);
			  ptMethod.setNearestLat(48.5);
			  ptMethod.setExtreme(7);
			  break;

     case 7: /*	Fixed Ishaa Interval (always 90) */
			  ptMethod.setFajrAngle(19.5);
			  ptMethod.setIshaaAngle(0.0);
			  ptMethod.setImsaakAngle(1.5);
			  ptMethod.setFajrInv(0);
			  ptMethod.setIshaaInv(90);
			  ptMethod.setImsaakInv(0);
			  ptMethod.setMathhab(1);
			  ptMethod.setNearestLat(48.5);
			  ptMethod.setExtreme(7);
			  break;
     }
}

/* Obtaining the direction of the shortest distance towards Qibla by
 * using the great circle formula
 */
public double getNorthQibla(PrayerTimeLocation ptLoc)
{
     double num;
	 double denom;
     num = Math.sin (deg2Rad (ptLoc.getDegreeLong() -  KAABA_LONG));
     denom = (Math.cos (deg2Rad (ptLoc.getDegreeLat())) * Math.tan (deg2Rad (KAABA_LAT))) -
	  (Math.sin (deg2Rad (ptLoc.getDegreeLat())) * ((Math.cos ((deg2Rad (ptLoc.getDegreeLong() -
							KAABA_LONG))))));
     return (Math.atan2 (num, denom) / deg2Rad(0));

}
}